import QtQuick 2.0
import Sailfish.Silica 1.0
import org.nemomobile.configuration 1.0
import Sailfish.Telephony 1.0
import MeeGo.QOfono 0.2
import com.jolla.gallery.ambience 1.0

Page {
    id: page
	
	OfonoManager { id: ofonoManager }
	
    ConfigurationGroup {
        id: statusareaSettings
        path: "/desktop/statusarea/settings"
		property int top_level: 70
        property int bottom_level: 30
		property int style: 2
		property bool volume_icon: true
		property bool presence_icon: false
		property bool date_peek: true
		property bool show_all_icons: false
		property bool custom_operator_name: false
		property bool dalvik: false
		property string operator_name: operatorNameInput.text		
		property int position: 0
		property int clock_size: 0		
		property int date_size: 0
        property bool lock_peek: true
		property string sim1_name: sim1NameInput.text
		property bool statusbar_gradient: false
		property int gradient_opacity: 10
		property bool colored_icons: false
		property int gradient_color: 1
		property int gradient_color_: 1
		property bool media_volume_icon: true		
		property int media_top_level: 70
        property int media_bottom_level: 30	
		property int media_style: 1
		property bool battery: false
    }

    SilicaFlickable {
        anchors.fill: parent
        contentHeight: content.height
        interactive: contentHeight > height

        Column {
		    id: content
            width: parent.width
            spacing: Theme.paddingLarge

            PageHeader {
			//% "Statusbar settings"
                title: qsTrId("Statusbar settings")
            }
	      			
        	ExpandingSection {
                buttonHeight: Theme.itemSizeMedium
                title: qsTr("Indicators settings")
        		visible: statusareaSettings.expand
                content.sourceComponent: Column {
        	        width: parent.width	
					
					TextSwitch {
						width: parent.width				
						text: qsTr("Show all statusbar icons on lockscreen")
						checked: statusareaSettings.show_all_icons
						onClicked: statusareaSettings.show_all_icons = checked
						description: qsTr("Bluetooth and GPS icons are hidden on lockscreen by default, enable this switch if you want see them")
					}
        		
					TextSwitch {
						width: parent.width				
						text: qsTr("Colored statusbar icons")
						description: qsTr("Enables colored overlay for statusbar content. This will be useful with light ambiences.")
						checked: statusareaSettings.colored_icons
						onClicked: statusareaSettings.colored_icons = checked			
					}
					
					TextSwitch {
						width: parent.width				
						text: qsTr("Show presence icon")
						checked: statusareaSettings.presence_icon
						onClicked: statusareaSettings.presence_icon = checked
						description: qsTr("Show presence indicator in statusbar or under clock. Some devices with community-ports don't have this indicator, please, contact with port-author")
					}
										
					ComboBox {
						width: parent.width
						visible: statusareaSettings.presence_icon
						label: qsTr("Choose indicator position")
						currentIndex: updateIndex(statusareaSettings.position)

						menu: ContextMenu {
							MenuItem {
								text: qsTr("Statusbar")
								onClicked: {
									statusareaSettings.position = 0
								}
							}
							
							MenuItem {
								text: qsTr("Clock bottom")
								onClicked: {
									statusareaSettings.position = 1
								}
							}
						}

						function updateIndex(value) {
							if (value == 0) {
								return 0
							} else if (value == 1) {
								return 1
							}
						}
					}
					
					TextSwitch {
						width: parent.width				
						text: qsTr("Hide battery indicator")
						checked: statusareaSettings.battery
						onClicked: statusareaSettings.battery = checked
						description: qsTr("This can be useful if you use Battery Overlay 2 app")
					}
					
					TextSwitch {
						width: parent.width				
						text: qsTr("Show AlienDalvik indicator")
						checked: statusareaSettings.dalvik
						onClicked: statusareaSettings.dalvik = checked
						description: qsTr("Show indicator if Android™-support is running")
					}	
					
        			TextSwitch {
        				width: parent.width
        				text: qsTr("Show ringing volume indicator")
        				description: qsTr("Show different icons according current ringing volume")
        				checked: statusareaSettings.volume_icon
        				onClicked: statusareaSettings.volume_icon = checked
        			}
        					
                    Slider {
                		visible: statusareaSettings.volume_icon
                		id: bottom
                        width: parent.width
                        label: qsTr("Medium level indicator bottom limit")
                        maximumValue: 70
                        minimumValue: 10
                        stepSize: 10
                        value: statusareaSettings.bottom_level
                        valueText: value + "%"
                
                        onValueChanged: statusareaSettings.bottom_level = Math.round(value)
                        onPressAndHold: cancel()
                    }
                			
                    Slider {
                		id: top
                		visible: statusareaSettings.volume_icon
                        width: parent.width
                        label: qsTr("Medium level indicator top limit")
                        maximumValue: 90
                        minimumValue: statusareaSettings.bottom_level
                        stepSize: 10
                        value: statusareaSettings.top_level
                        valueText: value + "%"
                
                        onValueChanged: statusareaSettings.top_level = Math.round(value)
                        onPressAndHold: cancel()
                    }
                									
                	Label {
                		visible: statusareaSettings.volume_icon
                        text: qsTr("Choose icon style")
                        width: parent.width
                		wrapMode: Text.Wrap
                		horizontalAlignment: Text.AlignHCenter
                    }
                			
                    Row {
                        width: parent.width
                		visible: statusareaSettings.volume_icon
                		anchors {
                			left: parent.left
                		}
                				
                		Rectangle {
                			id: style1
                			color: (statusareaSettings.style == 1) ? Theme.secondaryHighlightColor : "transparent" 
                			width: parent.width/3
                			height: parent.width/3
                					
                			Image {
                				source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-speaker-level-medium-1.svg"
                				width: Theme.iconSizeSmall
                				height: Theme.iconSizeSmall
                				anchors {
                					verticalCenter: parent.verticalCenter
                					horizontalCenter: parent.horizontalCenter
                				}
                			}
                					
                            MouseArea {
                                anchors.fill: parent
                                onClicked: { 
                					statusareaSettings.style = 1
                				}
                            }				
                		}	
                
                		Rectangle {
                			id: style2
                			color: (statusareaSettings.style == 2) ? Theme.secondaryHighlightColor : "transparent" 
                			width: parent.width/3
                			height: parent.width/3
                					
                			Image {
                				source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-speaker-level-medium-2.svg"
                				width: Theme.iconSizeSmall
                				height: Theme.iconSizeSmall
                				anchors {
                					verticalCenter: parent.verticalCenter
									horizontalCenter: parent.horizontalCenter
                				}
                			}
                					
                            MouseArea {
                                anchors.fill: parent
                                onClicked: { 
                					statusareaSettings.style = 2
                				}
                            }				
                		}

                		Rectangle {
                			id: style3
                			color: (statusareaSettings.style == 3) ? Theme.secondaryHighlightColor : "transparent" 
                			width: parent.width/3
                			height: parent.width/3
                					
                			Image {
                				source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-speaker-level-medium-3.svg"
                				width: Theme.iconSizeSmall
                				height: Theme.iconSizeSmall
                				anchors {
                					verticalCenter: parent.verticalCenter
									horizontalCenter: parent.horizontalCenter
                				}
                			}
                					
                            MouseArea {
                                anchors.fill: parent
                                onClicked: { 
                					statusareaSettings.style = 3
                				}
                            }				
                		}						
                	}
                			
                	Item {
                		width: 1
                		height: Theme.paddingSmall
                	}
                			
                    Label {
                		visible: statusareaSettings.volume_icon
                        width: parent.width
                        wrapMode: Text.Wrap
                		x: Theme.paddingSmall
                        text: qsTr("You will see these icons:")
                    }
                            			
                	Row {
                		spacing: Theme.paddingSmall
                		x: Theme.paddingSmall
                		visible: statusareaSettings.volume_icon
                				
                	    Image {
                            source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-speaker-level-zero-" + statusareaSettings.style + ".svg"
                		    anchors.verticalCenter: parent.verticalCenter
                        }
                				
                        Label {			
                            text: " = 0%"
                        }				
                    }
                			
                	Row {
                		spacing: Theme.paddingSmall
                		x: Theme.paddingSmall
                		visible: statusareaSettings.volume_icon
                				
                        Label {			
                            text: "> 0% "
                        }
                				
                	    Image {
                            source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-speaker-level-low-" + statusareaSettings.style + ".svg"
                		    anchors.verticalCenter: parent.verticalCenter
                        }	
                				
                        Label {			
                            text: " < " + bottom.value + "%"
                        }				
                    }
                								
                    Row {
                		spacing: Theme.paddingSmall
                		x: Theme.paddingSmall
                		visible: statusareaSettings.volume_icon
                				
                        Label {			
                            text: ">= " + bottom.value + "%"
                        }	
                
                	    Image {
                            source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-speaker-level-medium-" + statusareaSettings.style + ".svg"
                		    anchors.verticalCenter: parent.verticalCenter
                        }
                				
                        Label {			
                            text: " < " + top.value + "%"
                        }				
                    }
                
                    Row {
                		spacing: Theme.paddingSmall
                		x: Theme.paddingSmall
                		visible: statusareaSettings.volume_icon
                			
                	    Image {
                            source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-speaker-level-max-" + statusareaSettings.style + ".svg"
                		    anchors.verticalCenter: parent.verticalCenter
                        }	
                				
                        Label {			
                            text: " >= " + top.value + "%"
                        }			
                    }
        					
                	Item {
                		width: 1
                		height: Theme.paddingSmall
                	}
        					
        			Separator {
        				color: Theme.secondaryHighlightColor
        				width: parent.width
        				anchors.horizontalCenter: parent.horizontalCenter
        				horizontalAlignment: Qt.AlignHCenter
        			}
        					
        			TextSwitch {
        				width: parent.width
        				text: qsTr("Show media volume icon")
        				description: qsTr("Show different icons according current media volume")
        				checked: statusareaSettings.media_volume_icon
        				onClicked: statusareaSettings.media_volume_icon = checked
        			}
        					
                    Slider {
                		visible: statusareaSettings.media_volume_icon
                		id: bottom_media
                        width: parent.width
                        label: qsTr("Medium level indicator bottom limit")
                        maximumValue: 70
                        minimumValue: 10
                        stepSize: 5
                        value: statusareaSettings.media_bottom_level
                        valueText: value + "%"
                
                        onValueChanged: statusareaSettings.media_bottom_level = Math.round(value)
                        onPressAndHold: cancel()
                    }	
        					
                    Slider {
                		id: top_media
                		visible: statusareaSettings.media_volume_icon
                        width: parent.width
                        label: qsTr("Medium level indicator top limit")
                        maximumValue: 90
                        minimumValue: statusareaSettings.media_bottom_level
                        stepSize: 5
                        value: statusareaSettings.media_top_level
                        valueText: value + "%"
                
                        onValueChanged: statusareaSettings.media_top_level = Math.round(value)
                        onPressAndHold: cancel()
                    }
					
					Label {
                		visible: statusareaSettings.media_volume_icon
                        text: qsTr("Choose icon style")
                        width: parent.width
                		wrapMode: Text.Wrap
                		horizontalAlignment: Text.AlignHCenter
                    }
                			
                    Row {
                        width: parent.width
                		visible: statusareaSettings.media_volume_icon
                		anchors {
                			left: parent.left
                		}
                				
                		Rectangle {
                			id: media_style1
                			color: (statusareaSettings.media_style == 1) ? Theme.secondaryHighlightColor : "transparent" 
                			width: parent.width/3
                			height: parent.width/3
                					
                			Image {
                				source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-media-level-medium-1.svg"
                				width: Theme.iconSizeSmall
                				height: Theme.iconSizeSmall
                				anchors {
                					verticalCenter: parent.verticalCenter
                					horizontalCenter: parent.horizontalCenter
                				}
                			}
                					
                            MouseArea {
                                anchors.fill: parent
                                onClicked: { 
                					statusareaSettings.media_style = 1
                				}
                            }				
                		}	
                
                		Rectangle {
                			id: media_style2
                			color: (statusareaSettings.media_style == 2) ? Theme.secondaryHighlightColor : "transparent" 
                			width: parent.width/3
                			height: parent.width/3
                					
                			Image {
                				source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-media-level-medium-2.svg"
                				width: Theme.iconSizeSmall
                				height: Theme.iconSizeSmall
                				anchors {
                					verticalCenter: parent.verticalCenter
									horizontalCenter: parent.horizontalCenter
                				}
                			}
                					
                            MouseArea {
                                anchors.fill: parent
                                onClicked: { 
                					statusareaSettings.media_style = 2
                				}
                            }				
                		}

                		Rectangle {
                			id: media_style3
                			color: (statusareaSettings.media_style == 3) ? Theme.secondaryHighlightColor : "transparent" 
                			width: parent.width/3
                			height: parent.width/3
                					
                			Image {
                				source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-media-level-medium-3.svg"
                				width: Theme.iconSizeSmall
                				height: Theme.iconSizeSmall
                				anchors {
                					verticalCenter: parent.verticalCenter
									horizontalCenter: parent.horizontalCenter
                				}
                			}
                					
                            MouseArea {
                                anchors.fill: parent
                                onClicked: { 
                					statusareaSettings.media_style = 3
                				}
                            }				
                		}						
                	}
					
                	Item {
                		width: 1
                		height: Theme.paddingSmall
                	}
                			
                    Label {
                		visible: statusareaSettings.media_volume_icon
                        width: parent.width
                        wrapMode: Text.Wrap
                		x: Theme.paddingSmall
                        text: qsTr("You will see these icons:")
                    }
                            			
                	Row {
                		spacing: Theme.paddingSmall
                		x: Theme.paddingSmall
                		visible: statusareaSettings.media_volume_icon
                				
                	    Image {
                            source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-media-level-zero-" + statusareaSettings.media_style + ".svg"
                		    anchors.verticalCenter: parent.verticalCenter
                        }
                				
                        Label {			
                            text: " = 0%"
                        }				
                    }
                			
                	Row {
                		spacing: Theme.paddingSmall
                		x: Theme.paddingSmall
                		visible: statusareaSettings.media_volume_icon
                				
                        Label {			
                            text: "> 0% "
                        }
                				
                	    Image {
                            source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-media-level-low-" + statusareaSettings.media_style + ".svg"
                		    anchors.verticalCenter: parent.verticalCenter
                        }	
                				
                        Label {			
                            text: " < " + bottom_media.value + "%"
                        }				
                    }
                								
                    Row {
                		spacing: Theme.paddingSmall
                		x: Theme.paddingSmall
                		visible: statusareaSettings.media_volume_icon
                				
                        Label {			
                            text: ">= " + bottom_media.value + "%"
                        }	
                
                	    Image {
                            source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-media-level-medium-" + statusareaSettings.media_style + ".svg"
                		    anchors.verticalCenter: parent.verticalCenter
                        }
                				
                        Label {			
                            text: " < " + top_media.value + "%"
                        }				
                    }
                
                    Row {
                		spacing: Theme.paddingSmall
                		x: Theme.paddingSmall
                		visible: statusareaSettings.media_volume_icon
                			
                	    Image {
                            source: "/usr/share/patchmanager/patches/ultimate-statusbar/icon-m-media-level-max-" + statusareaSettings.media_style + ".svg"
                		    anchors.verticalCenter: parent.verticalCenter
                        }	
                				
                        Label {			
                            text: " >= " + top_media.value + "%"
                        }			
                    }					
                }				
            }
						
            Separator {
                color: Theme.secondaryHighlightColor
                width: parent.width
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Qt.AlignHCenter
            }	
			
            TextSwitch {
                width: parent.width				
                text: qsTr("Show statusbar on Sneak Peek")
                checked: statusareaSettings.lock_peek
                onClicked: statusareaSettings.lock_peek = checked
				description: qsTr("Show statusbar in addition to other information when taking your phone out of pocket, for example")
            }
						
            TextSwitch {
                width: parent.width				
                text: qsTr("Enable date when peeking")
                checked: statusareaSettings.date_peek
                onClicked: statusareaSettings.date_peek = checked
				description: qsTr("Show date under clock when you're peeking")
            }
			
            ComboBox {
                width: parent.width
				visible: statusareaSettings.date_peek
                label: qsTr("Choose date size")
                currentIndex: updateIndex(statusareaSettings.position)

                menu: ContextMenu {
                    MenuItem {
                        text: qsTr("Medium (default)")
                        onClicked: {
							statusareaSettings.date_size = 0
                        }
                    }
                    MenuItem {
                        text: qsTr("Small")
                        onClicked: {
							statusareaSettings.date_size = 1
                        }
                    }
                }

                function updateIndex(value) {
                    if (value == 0) {
                        return 0
                    } else if (value == 1) {
                        return 1
                    }
                }
            }
									
            TextSwitch {
                width: parent.width				
                text: qsTr("Custom operator name")
				visible: ofonoManager.modems.length > 0
                checked: statusareaSettings.custom_operator_name
                onClicked: statusareaSettings.custom_operator_name = checked
            }	
			
            TextField {
                id: operatorNameInput
				visible: statusareaSettings.custom_operator_name && !Telephony.multiSimSupported
                width: parent.width
                placeholderText: qsTr("Enter your operator name here")
                label: qsTr("Don't enter too long name")
                text: statusareaSettings.operator_name
                onTextChanged: statusareaSettings.operator_name = text
            }	
			
			TextField {
                id: sim1NameInput
				visible: statusareaSettings.custom_operator_name && Telephony.multiSimSupported
                width: parent.width
                placeholderText: qsTr("SIM1 | SIM2")
                label: qsTr("Don't enter too long name")
                text: statusareaSettings.sim1_name
                onTextChanged: statusareaSettings.sim1_name = text
            }
						
			TextSwitch {
                width: parent.width				
                text: qsTr("Statusbar gradient background")
				description: qsTr("Enables colored gradient background with customizable opacity and color. This will be useful with light ambiences.")
                checked: statusareaSettings.statusbar_gradient
                onClicked: statusareaSettings.statusbar_gradient = checked			
			}
			
			ComboBox {
				width: parent.width
				visible: statusareaSettings.statusbar_gradient
				label: qsTr("Gradient color")
				currentIndex: updateIndex(statusareaSettings.gradient_color_)

				menu: ContextMenu {
					MenuItem {
						text: qsTr("Black")
						onClicked: {
							statusareaSettings.gradient_color_ = 0
						}
					}	
		
					MenuItem {
						text: qsTr("Theme color")
						onClicked: {
							statusareaSettings.gradient_color_ = 1
						}
					}
				
					MenuItem {
						text: qsTr("Custom")
						onClicked: {
							statusareaSettings.gradient_color_ = 2
						}
					}
				}
				
				function updateIndex(value) {
					if (value == 0) {
						return 0
					} else if (value == 1) {
						return 1
					} else if (value == 2) {
						return 2
					}
				}
			}
			
            Slider {
				id: slider			
				visible: statusareaSettings.statusbar_gradient && statusareaSettings.gradient_color_ == 2
			    Rectangle {
					id: background
					x: slider.leftMargin
					z: -1
					width: slider._grooveWidth
			        height: Theme.paddingMedium
			        anchors.top: parent.verticalCenter
					anchors.topMargin: -Theme.paddingLarge*2
			
			        ShaderEffect {
			            id: rainbow
			            property variant src: background
			            property real saturation: 1.0
			            property real lightness: 0.5
			            property real alpha: 1.0
			
			            width: parent.width
			            height: parent.height
			
			            // Fragment shader to create hue color wheel background
			            fragmentShader: "
			                varying highp vec2 coord;
			                varying highp vec2 qt_TexCoord0;
			                uniform sampler2D src;
			                uniform lowp float qt_Opacity;
			                uniform lowp float saturation;
			                uniform lowp float lightness;
			                uniform lowp float alpha;
			
			                void main() {
			                    float r, g, b;
			
			                    float h = qt_TexCoord0.x * 360.0;
			                    float s = saturation;
			                    float l = lightness;
			
			                    float c = (1.0 - abs(2.0 * l - 1.0)) * s;
			                    float hh = h / 60.0;
			                    float x = c * (1.0 - abs(mod(hh, 2.0) - 1.0));
			
			                    int i = int( hh );
			
			                    if (i == 0) {
			                        r = c; g = x; b = 0.0;
			                    } else if (i == 1) {
			                        r = x; g = c; b = 0.0;
			                    } else if (i == 2) {
			                        r = 0.0; g = c; b = x;
			                    } else if (i == 3) {
			                        r = 0.0; g = x; b = c;
			                    } else if (i == 4) {
			                        r = x; g = 0.0; b = c;
			                    } else if (i == 5) {
			                        r = c; g = 0.0; b = x;
			                    } else {
			                        r = 0.0; g = 0.0; b = 0.0;
			                    }
			
			                    float m = l - 0.5 * c;
			
			                    lowp vec4 tex = texture2D(src, qt_TexCoord0);
			                    gl_FragColor = vec4(r+m,g+m,b+m,alpha) * qt_Opacity;
			                }"
			        }
			    }	
				
                width: parent.width
                minimumValue: 0
                maximumValue: 100
                stepSize: 1
                value: statusareaSettings.gradient_color
				valueText: "|"
                onValueChanged: statusareaSettings.gradient_color = value
                onPressAndHold: cancel()
				
				Label {
					width: parent.width
					wrapMode: Text.Wrap
					font.pixelSize: Theme.fontSizeSmall
					horizontalAlignment: Text.AlignHCenter
                    anchors.top: parent.verticalCenter
                    anchors.topMargin: Theme.paddingMedium*2
					color: Color.toHighlight(Color.fromHsva(statusareaSettings.gradient_color/100, 1.0, 0.5, 1.0))
					text: qsTr("Gradient color")
				}						
            }
			
            Slider {
        		visible: statusareaSettings.statusbar_gradient
                width: parent.width
                label: qsTr("Gradient opacity")
                maximumValue: 100
                minimumValue: 10
                stepSize: 10
                value: statusareaSettings.gradient_opacity
                valueText: value + "%"
        
                onValueChanged: statusareaSettings.gradient_opacity = Math.round(value)
                onPressAndHold: cancel()
            }
			
			ComboBox {
				width: parent.width

				label: qsTr("Statusbar clock size")
				currentIndex: updateIndex(statusareaSettings.clock_size)

				menu: ContextMenu {
					MenuItem {
						text: qsTr("Default")
						onClicked: {
							statusareaSettings.clock_size = 0
						}
					}	
		
					MenuItem {
						text: qsTr("Medium")
						onClicked: {
							statusareaSettings.clock_size = 1
						}
					}
				
					MenuItem {
						text: qsTr("Large")
						onClicked: {
							statusareaSettings.clock_size = 2
						}
					}
				}
				
				function updateIndex(value) {
					if (value == 0) {
						return 0
					} else if (value == 1) {
						return 1
					} else if (value == 2) {
						return 2
					}
				}
			}	
	
            Separator {
                color: Theme.secondaryHighlightColor
                width: parent.width
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Qt.AlignHCenter
            }
			
            Label {
                width: parent.width
                wrapMode: Text.Wrap
                horizontalAlignment: Text.AlignHCenter
    			text: qsTr("Big thanks to Francis B. Belisle for awesome SVG-icons :)")
            }
			
            Label {
                width: parent.width
                wrapMode: Text.Wrap
                horizontalAlignment: Text.AlignHCenter
                text: qsTr("Created by Ancelad in 0x7E0")
            }
						
		    Label {
                text: qsTr("Donations are welcome :)")
                width: parent.width
				wrapMode: Text.Wrap
                horizontalAlignment: Text.AlignHCenter
            }

            Button {
                text: qsTr("Donate")
                width: text.width
                anchors.horizontalCenter: parent.horizontalCenter
                onClicked: {
                    Qt.openUrlExternally("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=E33PBZ79WTQJY")
                }
            }			
        }
    }
}
